@@ -1,10 +1,13 @@
 package Inline::Java::Array ;
 @Inline::Java::Array::ISA = qw(Inline::Java::Array::Tie) ;
 
+
 use strict ;
+
+$Inline::Java::Array::VERSION = '0.20' ;
+
 use Carp ;
 
-$Inline::Java::Array::VERSION = '0.53_90' ;
 
 # Here we store as keys the knots and as values our blessed objects
 my $OBJECTS = {} ;
@@ -20,8 +23,8 @@ sub new {
 
 	$OBJECTS->{$knot} = $object ;
 
-	Inline::Java::debug(5, "this = '$this'") ; 
-	Inline::Java::debug(5, "knot = '$knot'") ; 
+	Inline::Java::debug("this = $this") ; 
+	Inline::Java::debug("knot = $knot") ; 
 
 	return $this ;
 }
@@ -34,7 +37,7 @@ sub __get_object {
 
 	my $ref = $OBJECTS->{$knot} ;
 	if (! defined($ref)){
-		croak "Unknown Java array reference '$knot'" ;
+		croak "Unknown Java array reference $knot" ;
 	}
 	
 	return $ref ;
@@ -56,15 +59,7 @@ sub length {
 
 	my $ret = undef ;
 	eval {
-		# Check the cached value
-		$ret = $obj->__get_private()->{array_length} ;
-		if (! defined($ret)){
-			$ret = $obj->__get_private()->{proto}->CallJavaMethod('getLength', [], []) ;
-			$obj->__get_private()->{array_length} = $ret ;
-		}
-		else{
-			Inline::Java::debug(4, "using cached array length $ret") ;
-		}
+		$ret = $obj->__get_private()->{proto}->CallJavaMethod('getLength', [], []) ;
 	} ;
 	croak $@ if $@ ;
 
@@ -109,7 +104,7 @@ sub __set_element {
 	# the array.
 	my $java_class = $obj->__get_private()->{java_class} ;
 	my $elem_class = $java_class ;
-	my $an = Inline::Java::Array::Normalizer->new($obj->__get_private()->{inline}, $java_class) ;
+	my $an = new Inline::Java::ArrayNorm($java_class) ;
 	if ($an->{req_nb_dim} > 1){
 		$elem_class =~ s/^\[// ;
 	}
@@ -119,7 +114,7 @@ sub __set_element {
 
 	my $ret = undef ;
 	eval {
-		my ($new_args, $score) = Inline::Java::Class::CastArguments([$s], [$elem_class], $obj->__get_private()->{inline}) ;
+		my ($new_args, $score) = Inline::Java::Class::CastArguments([$s], [$elem_class], $obj->__get_private()->{module}) ;
 		$ret = $obj->__get_private()->{proto}->SetJavaMember($idx, [$elem_class], $new_args) ;
 	} ;
 	croak $@ if $@ ;
@@ -138,7 +133,9 @@ sub AUTOLOAD {
 	# method.
 	$func_name =~ s/^(.*)::// ;
 
-	croak "Can't call method '$func_name' on Java arrays" ;
+	Inline::Java::debug("$func_name") ;
+
+	croak "Can't call method $func_name on Java arrays" ;
 }
 
 
@@ -147,12 +144,14 @@ sub DESTROY {
 
 	my $knot = tied @{$this} ;
 	if (! $knot){
-		Inline::Java::debug(4, "destroying Inline::Java::Array::Tie") ;
+		Inline::Java::debug("Destroying Inline::Java::Array::Tie") ;
 
 		$OBJECTS->{$this} = undef ;
 	}
 	else{
-		Inline::Java::debug(4, "destroying Inline::Java::Array") ;
+		# Here we can't untie because we still have a reference in $OBJECTS
+		# untie @{$this} ;
+		Inline::Java::debug("Destroying Inline::Java::Array") ;
 	}
 }
 
@@ -293,12 +292,11 @@ use Carp ;
 
 sub new {
 	my $class = shift ;
-	my $inline = shift ;
 	my $java_class = shift ;
 	my $ref = shift ;
 
 	if (! Inline::Java::Class::ClassIsArray($java_class)){
-		croak "Can't create Java array of non-array class '$java_class'" ;
+		croak "Can't create Java array of non-array class $java_class" ;
 	}
 
 	my $this = {} ;
@@ -307,8 +305,6 @@ sub new {
 	$this->{map} = {} ;
 	$this->{ref} = $ref ;
 	$this->{array} = [] ;
-	$this->{score} = 0 ;
-	$this->{inline} = $inline ;
 	
 	bless ($this, $class) ;
 
@@ -341,7 +337,6 @@ sub InitFromArray {
 
 sub InitFromFlat {
 	my $this = shift ;
-	my $inline = shift ;
 	my $dims = shift ;
 	my $list = shift ;
 	my $level = shift ;
@@ -373,8 +368,8 @@ sub InitFromFlat {
 
 			my @dims = @{$dims} ;
 			shift @dims ;
-			my $obj = Inline::Java::Array::Normalizer->new($inline, $java_class) ;
-			$obj->InitFromFlat($inline, \@dims, \@sub, $level + 1) ;
+			my $obj = new Inline::Java::Array::Normalizer($java_class) ;
+			$obj->InitFromFlat(\@dims, \@sub, $level + 1) ;
 			$elem = $obj->{array} ;
 		}
 		my $nb = scalar(@{$this->{array}}) ;
@@ -412,7 +407,7 @@ sub AnalyzeArrayClass {
 
 	my $pclass = $map{$type} ;
 	if (! $pclass){
-		croak "Can't determine array type for '$java_class'" ;
+		croak "Can't determine array type for $java_class" ;
 	}
 
 	$this->{req_element_class} = $pclass ;
@@ -432,19 +427,16 @@ sub ValidateArray {
 	my $array = shift ;
 	my $level = shift || 0 ;
 
-	if (! ((defined($ref))&&(UNIVERSAL::isa($ref, "ARRAY")))){
+	if (! UNIVERSAL::isa($ref, "ARRAY")){
 		# We must start with an array of some kind...
-		croak "'$ref' is not an array reference" ;
+		croak "$ref is not an array reference" ;
 	}
 
-	my $map = $this->{map} ;
-	if (! exists($map->{$level}->{max})){
-		$map->{$level}->{max} = 0 ;
-	}
 	$this->ValidateElements($ref, $array, $level) ;
 
+	my $map = $this->{map} ;
 	foreach my $elem (@{$ref}){
-		if ((defined($elem))&&(UNIVERSAL::isa($elem, "ARRAY"))){
+		if (UNIVERSAL::isa($elem, "ARRAY")){
 			# All the elements at this level are sub-arrays.
 			my $sarray = [] ;
 			$this->ValidateArray($elem, $sarray, $level + 1) ;
@@ -462,13 +454,13 @@ sub ValidateArray {
 		my @dims = () ;
 		my $max_cells = 1 ;
 		foreach my $l (@levels){
-			push @dims, $map->{$l}->{max} ;
+			push @dims, ($map->{$l}->{max} || 0) ;
 			$max_cells *= $map->{$l}->{max} ;
 		}
 		my $nb_cells = ($map->{$last}->{count} || 0) ;
-		Inline::Java::debug(4, "array is [" . join("][", @dims) . "]") ;
-		Inline::Java::debug(4, "array has         $nb_cells declared cells") ;
-		Inline::Java::debug(4, "array should have $max_cells declared cells") ;
+		Inline::Java::debug("array is [" . join("][", @dims) . "]") ;
+		Inline::Java::debug("array has         $nb_cells declared cells") ;
+		Inline::Java::debug("array should have $max_cells declared cells") ;
 		$this->{dim} = \@dims ;
 		$this->{nb_dim} = scalar(@dims) ;
 
@@ -490,8 +482,10 @@ sub ValidateElements {
 	my $level = shift ;
 
 	my $map = $this->{map} ;
+
 	my $cnt = scalar(@{$ref}) ;
-	my $max = $map->{$level}->{max} ;
+	my $max = $map->{$level}->{max} || 0 ;
+
 	if ($cnt > $max){
 		$map->{$level}->{max} = $cnt ;
 	}
@@ -506,9 +500,8 @@ sub ValidateElements {
 				(UNIVERSAL::isa($elem, "Inline::Java::Object"))||
 				(! ref($elem))){
 				$this->CheckMap("BASE_ELEMENT", $level) ;
-				my @ret = $this->CastArrayArgument($elem) ;
-				$array->[$i] = $ret[0] ;
-				$this->{score} += $ret[1] ;
+				$elem = $this->CastArrayArgument($elem) ;
+				$array->[$i] = $elem ;
 			}
 			else{
 				croak "A Java array can only contain scalars, Java objects or array references" ;
@@ -516,7 +509,8 @@ sub ValidateElements {
 		}
 		$map->{$level}->{count}++ ;
 	}
-}	
+}
+
 
 sub CheckMap {
 	my $this = shift ;
@@ -540,9 +534,9 @@ sub CastArrayArgument {
 
 	my $element_class = $this->{req_element_class} ;
 
-	my ($new_arg, $score) = Inline::Java::Class::CastArgument($arg, $element_class, $this->{inline}) ;
+	my ($new_arg, $score) = Inline::Java::Class::CastArgument($arg, $element_class) ;
 
-	return ($new_arg, $score) ;
+	return $new_arg ;
 }
 
 
@@ -557,9 +551,7 @@ sub FillArray {
 	my $max = $map->{$level}->{max} ;
 	my $nb = scalar(@{$array}) ;
 
-	my $type = $map->{$level}->{type} ;
-	# Type can be undefined when array is zero length.
-	if ((defined($type))&&($type eq "SUB_ARRAY")){
+	if ($map->{$level}->{type} eq "SUB_ARRAY"){
 		foreach my $elem (@{$array}){
 			if (! defined($elem)){
 				$elem = [] ;
@@ -571,7 +563,7 @@ sub FillArray {
 	if ($nb < $max){
 		# We must stuff...
 		for (my $i = $nb ; $i < $max ; $i++){
-			if ((defined($type))&&($type eq "SUB_ARRAY")){
+			if ($map->{$level}->{type} eq "SUB_ARRAY"){
 				my $elem = [] ;
 				$this->FillArray($elem, $level + 1) ;
 				push @{$array}, $elem ;
@@ -616,7 +608,7 @@ sub MakeElementList {
 
 	my @ret = () ;
 	foreach my $elem (@{$array}){
-		if ((defined($elem))&&(UNIVERSAL::isa($elem, "ARRAY"))){
+		if (UNIVERSAL::isa($elem, "ARRAY")){
 			my $sret = $this->MakeElementList($elem) ;
 			push @ret, @{$sret} ;			
 		}
@@ -631,4 +623,112 @@ sub MakeElementList {
 }
 
 
+
+package Inline::Java::Array ;
+
+
 1 ; 
+
+
+__DATA__
+
+
+class InlineJavaArray {
+	InlineJavaServer ijs ;
+	InlineJavaClass ijc ;
+
+
+	InlineJavaArray(InlineJavaServer _ijs, InlineJavaClass _ijc){
+		ijs = _ijs ;
+		ijc = _ijc ;
+	}
+
+
+	Object CreateArray(Class c, StringTokenizer st) throws InlineJavaException {
+		StringBuffer sb = new StringBuffer(st.nextToken()) ;
+		sb.replace(0, 1, "") ;
+		sb.replace(sb.length() - 1, sb.length(), "") ;
+
+		StringTokenizer st2 = new StringTokenizer(sb.toString(), ",") ;
+		ArrayList al = new ArrayList() ;
+		while (st2.hasMoreTokens()){
+			al.add(al.size(), st2.nextToken()) ;
+		}
+
+		int size = al.size() ;
+		int dims[] = new int[size] ;
+		for (int i = 0 ; i < size ; i++){
+			dims[i] = Integer.parseInt((String)al.get(i)) ;
+			ijs.debug("    array dimension: " + (String)al.get(i)) ;
+		}
+
+		Object array = null ;
+		try {
+			array = Array.newInstance(c, dims) ;
+
+			ArrayList args = new ArrayList() ;
+			while (st.hasMoreTokens()){
+				args.add(args.size(), st.nextToken()) ;
+			}
+
+			// Now we need to fill it. Since we have an arbitrary number
+			// of dimensions, we can do this recursively.
+
+			PopulateArray(array, c, dims, args) ;
+		}
+		catch (IllegalArgumentException e){
+			throw new InlineJavaException("Arguments to array constructor for class " + c.getName() + " are incompatible: " + e.getMessage()) ;
+		}
+
+		return array ;
+	}
+
+
+	void PopulateArray (Object array, Class elem, int dims[], ArrayList args) throws InlineJavaException {
+		if (dims.length > 1){
+			int nb_args = args.size() ;
+			int nb_sub_dims = dims[0] ;
+			int nb_args_per_sub_dim = nb_args / nb_sub_dims ;
+
+			int sub_dims[] = new int[dims.length - 1] ;
+			for (int i = 1 ; i < dims.length ; i++){
+				sub_dims[i - 1] = dims[i] ;
+			}
+	
+			for (int i = 0 ; i < nb_sub_dims ; i++){
+				// We want the args from i*nb_args_per_sub_dim -> 
+				ArrayList sub_args = new ArrayList() ; 
+				for (int j = (i * nb_args_per_sub_dim) ; j < ((i + 1) * nb_args_per_sub_dim) ; j++){
+					sub_args.add(sub_args.size(), (String)args.get(j)) ;
+				}
+				PopulateArray(((Object [])array)[i], elem, sub_dims, sub_args) ;
+			}
+		}
+		else{
+			String msg = "In creation of array of " + elem.getName() + ": " ;
+			try {
+				for (int i = 0 ; i < dims[0] ; i++){
+					String arg = (String)args.get(i) ;
+
+					Object o = ijc.CastArgument(elem, arg) ;
+					Array.set(array, i, o) ;
+					if (o != null){
+						ijs.debug("      setting array element " + String.valueOf(i) + " to " + o.toString()) ;
+					}
+					else{
+						ijs.debug("      setting array element " + String.valueOf(i) + " to " + o) ;
+					}
+		 		}
+			}
+			catch (InlineJavaCastException e){
+				throw new InlineJavaCastException(msg + e.getMessage()) ;
+			}
+			catch (InlineJavaException e){
+				throw new InlineJavaException(msg + e.getMessage()) ;
+			}
+		}
+	}
+}
+	
+	
+
